﻿Ext.define('App.Control.Kivii.Finances.Policy.Form', {
    extend: 'System.Control.EntityForm',
    entityType: 'Kivii.Finances.Entities.Policy',
    xtype: 'App.Control.Kivii.Finances.Policy.Form',
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 1000,
            large: 0
        }
    },
    // title: '开票信息',
    // iconCls: 'x-fa fa-file',
    tbar: ['->', {
        text: '保存',
        iconCls: 'x-fa fa-save',
        iconAlign: 'left',
        handler: '_onSaveEntity',
        bind: {
            disabled: '{!formIsValid}'
        }
    }],
    defaults: {
        labelWidth: 60,
        labelAlign: 'right',
        xtype: 'textfield',
        margin: '0 10 10 10',
        responsiveCls: 'big-100 small-100'
    },
    items: [{
        xtype: 'comboSelector',
        fieldLabel: '所属部门',
        responsiveCls: 'big-100 small-100',
        editable: false,
        allowBlank: false,
        dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
        reference: 'depart',
        keyField: 'Kvid',
        valueField: 'FullName',
        bind: {
            key: '{entity.OwnerKvid}',
            value: '{entity.OwnerName}'
        }
    }, {
        fieldLabel: '政策名称',
        bind: '{entity.Name}',
        responsiveCls: 'big-60 small-100',
        allowBlank: false
    }, {
        xtype: 'categoryField',
        fieldLabel: '政策类型',
        responsiveCls: 'big-40 small-100',
        allowBlank: false,
        internalCode: 'Type',
        bind: '{entity.Type}'
    }, {
        xtype: 'categoryField',
        fieldLabel: '政策折扣',
        responsiveCls: 'big-30 small-100',
        internalCode: 'DiscountRate',
        bind: '{entity.DiscountRate}'
    }, {
        xtype: 'numberfield',
        fieldLabel: '折扣金额',
        minValue: 0,
        responsiveCls: 'big-30 small-100',
        bind: '{entity.Amount}'
    },  {
        xtype: 'categoryField',
        fieldLabel: '货币单位',
        responsiveCls: 'big-40 small-100',
        allowBlank: false,
        internalCode: 'Currency',
        bind: '{entity.Currency}'
    },{
        xtype: 'datefield',
        format: 'Y-m-d',
        altFormats: 'D, d M Y H:i:s T|c|Y-m-d|Y-m-d H:i|Y/n/j',
        fieldLabel: '生效日期',
        editable: false,
        responsiveCls: 'big-40 small-100',
        allowBlank: false,
        bind: {
            value: '{entity.EffectiveTime}'
        }
    }, {
        xtype: 'datefield',
        format: 'Y-m-d',
        altFormats: 'D, d M Y H:i:s T|c|Y-m-d|Y-m-d H:i|Y/n/j',
        fieldLabel: '失效日期',
        editable: false,
        responsiveCls: 'big-60 small-100',
        triggers: {
            clear: {
                weight: 2,
                cls: 'x-fa fa-times',
                handler: '_cleanExpiredTime'
            }
        },
        bind: {
            value: '{entity.ExpiredTime}'
        }
    }, {
        fieldLabel: '摘&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;要',
        bind: {
            value: '{entity.Summary}'
        }
    }, {
        fieldLabel: '备&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;注',
        bind: {
            value: '{entity.Remark}'
        }
    }],
    controller: {
        _cleanExpiredTime: function(_ctrl) {
            _ctrl.setValue(null);
            let _controller = this;
            let _view = _controller.getView();
            let _vm=_controller.getViewModel();
            _vm.data.entity.ExpiredTime=null;
        },
        _onSaveEntity: function() {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            _view.saveEntity();
        }
    }
});