﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Statistic.Payment', {
    extend: 'System.Control.EntityList',
    xtype: 'App.View.Kivii.Finances.Statistic.Payment',
    entityType: 'Kivii.Finances.Entities.Payment',
    requires: ['System.Control.ComboSelector','App.Control.DownloadField'],
    dataSource: '/Restful/Kivii.Finances.Entities.Payment/Statistic.json',
    iconCls: 'x-fa fa-spinner',
    title: '到账统计',
    filter: {
        BeginTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
        OwnerKvid:KiviiContext.CurrentMember.DepartmentKvid,
    	// OwnerType:"Department",
        IsPayee: true
    },
    //在末尾保留足够的滚动条大小的控件
    reserveScrollbar:true,
    //默认情况下启用缓冲渲染
    //启用缓存会造成空白行的问题,有些数据过多会预留渲染控件,切换成数据少的会预留空白
    bufferedRenderer:false,
    viewModel: {
        data: {
            _selection: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'OperateTime',
            direction: 'DESC'
        }]
    },
    selModel: {
        selType: 'checkboxmodel',
        checkboxSelect: true,
        pruneRemoved: false,
        extensible: 'y'
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            margin: '0 0 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '全页 到账金额: {amount} &nbsp;&nbsp;已开票: {amountInvoice} &nbsp;&nbsp;已使用: {amountUsed}'
            },
            fieldStyle: {
                fontSize: '14px'
            }
        }]
    }, {
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'scroller',
        items: [{
            xtype: 'comboSelector',
            dataSource: '/Restful/Kivii.Organizations.Entities.Organization/Query.json',
            reference: 'depart',
            responsiveCls: 'big-100 small-100',
            keyField: 'Kvid',
            valueField: 'FullName',
            key: KiviiContext.CurrentMember.DepartmentKvid,
            value: KiviiContext.CurrentMember.DepartmentName,
            editable: false,
            width: 300,
            allowBlank: true,
            // triggers: {
            //     clear: {
            //         weight: 2,
            //         cls: 'x-fa fa-times',
            //         handler: '_cleanSelect'
            //     }
            // },
            listeners: {
                select: '_onSelect'
            }
        },{
                xtype: 'combo',
                queryMode: 'local',
                displayField: 'displayInfo',
                valueField: 'used',
                width: 130,
                editable: false,
                filterPickList: true,
                reference: 'used',
                store: {
                    fields: ['displayInfo', 'used'],
                    data: [{
                        displayInfo: '已使用',
                        used: true
                    }, {
                        displayInfo: '未使用',
                        used: false
                    }]
                },
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanUsed'
                    }
                },
                listeners: {
                    select: '_onSelectUsed'
                }
            }, {
                xtype: 'combo',
                queryMode: 'local',
                displayField: 'displayInfo',
                valueField: 'invoiced',
                width: 130,
                editable: false,
                filterPickList: true,
                reference: 'invoiced',
                store: {
                    fields: ['displayInfo', 'invoiced'],
                    data: [{
                        displayInfo: '已开票',
                        invoiced: true
                    }, {
                        displayInfo: '未开票',
                        invoiced: false
                    }]
                },
                triggers: {
                    clear: {
                        weight: 2,
                        cls: 'x-fa fa-times',
                        handler: '_cleanInvoiced'
                    }
                },
                listeners: {
                    select: '_onSelectInvoiced'
                }
            },{
                xtype: 'datefield',
                fieldLabel: '到账时间  开始',
                format: 'Y-m-d',
                labelWidth: 90,
                width: 220,
                reference: 'startTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            },
            {
                xtype: 'datefield',
                fieldLabel: '结束',
                format: 'Y-m-d',
                labelWidth: 30,
                width: 160,
                reference: 'endTime',
                editable: false,
                value: new Date(new Date().getFullYear(), new Date().getMonth(), new Date().getDate() + 1),
                listeners: {
                    select: '_onSelectMonth'
                }
            }, {
                xtype: 'searchField',
                flex: 1,
                minWidth: 250,
                labelWidth: 60,
                queryField: 'SerialNumber,PayerName,PayerAccountName,PayerAccountSerialNumber',
                // queryField: 'Amount',
                emptyText: '模糊搜索流水号、付款单位、收付款账号或名称',
            }, {
            xtype: 'downloadField',
            dataSource: '/Restful/Kivii.Finances.Entities.Payment/Statistic.xlsx', //数据源
            fileName: '数据导出',
            window: {
                type: 'App.Control.Kivii.Finances.General.Template.Selector',
                title: '选择模板',
                iconCls: 'x-fa fa-file-excel-o',
                autoScroll: true,
                width: 700,
                height: 350,
                closeAction: 'hide',
                modal: true,
                layout: 'fit',
                filter: {
	                FolderPath: '/Finances/Payment/Files',
	                TagsContains: 'Statistic'
	            }
            }
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: true
        },
        items: [{
            xtype: 'rownumberer',
            align: 'center',
            resizable: false
        }, {
            text: '流水号',
            dataIndex: 'SerialNumber',
            width: 120
        }, {
            text: '所属部门',
            dataIndex: 'OwnerName',
            renderer: '_renderOwnerName',
            width: 120
        }, {
            text: '付款单位',
            dataIndex: 'PayerName',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return '合计 ￥';
            },
            width: 220
        }, {
            text: '认领金额',
            align: 'right',
            dataIndex: 'Amount',
            renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 110
        }, {
            text: '开票金额',
            align: 'right',
            dataIndex: 'AmountInvoice',
            renderer: '_renderAmount',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 100
        }, {
            text: '已用金额',
            align: 'right',
            dataIndex: 'AmountUsed',
            renderer: '_renderAmount',
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('{0}', value.toFixed(2));
            },
            width: 100
        }, {
            text: '付款类型',
            dataIndex: 'Type',
            renderer: '_renderType',
            width: 100
        },  {
            xtype: 'datecolumn',
            text: '认领日期',
            format: 'Y-m-d H:i',
            dataIndex: 'CreateTime',
            width: 150
        }, {
            text: '认领人',
            dataIndex: 'OperatorName',
            width: 100
        }, {
            text: '摘要',
            dataIndex: 'Summary',
            width: 150
        },  {
            xtype: 'datecolumn',
            text: '到账日期',
            format: 'Y-m-d H:i',
            dataIndex: 'OperateTime',
            width: 150
        }, {
            text: '付款银行',
            dataIndex: 'PayerAccountName',
            width: 280
        }, {
            text: '付款账号',
            dataIndex: 'PayerAccountSerialNumber',
            width: 180
        }, {
            text: '备注',
            dataIndex: 'Remark',
            minWidth:200,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate',
        viewready: '_viewReady'
    },
    controller: {
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _viewReady: function() {
            let _me = this;
            let _view = _me.getView();
            let _vm = _me.getViewModel();
            let _store = _view.getStore();
            _store.on('load', function(_this, _records, _successful, _operation, _eOpts) {
            	let _rtns = Ext.decode(_operation.getResponse().responseText);
            	_vm.set('amount', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_rtns.TotalAmount.toFixed(2)));
                _vm.set('amountInvoice', KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_rtns.TotalAmountInvoice.toFixed(2)));
                _vm.set('amountUsed', KiviiContext.Const.FinancesFunctionsMoneyToFmoney((_rtns.TotalAmountUsed).toFixed(2)));
            });
        },
        _renderOwnerName: function(_value, _meta, _record, _rowIndex) {
            if (_value === null || _value === undefined || _value === '') {
                _meta.style = 'background-color:#FFC0CB';
            }
            return _value;
        },
        _renderAmount: function(_value, _meta, _record, _rowIndex) {
            if (_value === _record.data.Amount) {
                _meta.style = 'background-color:#B4EEB4';
            }
            return KiviiContext.Const.FinancesFunctionsMoneyToFmoney(_value);
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            _meta.style = 'background-color:#B4EEB4';
            if (_value === 'UnBiz') {
                _meta.style = 'background-color:#F0E68C';
                return '非业务';
            }
            if (_value === 'Refund') {
                _meta.style = 'background-color:#FFC0CB';
                return '退款';
            }
            if (_value === 'Split') {
                return '业务认领';
            }
            if (_value === 'Bank') {
                return '银行转账';
            }
            if (_value === 'Pos') {
                return '刷卡';
            }
            if (_value === 'Cash') {
                return '现金';
            }
            if (_value === 'AliPay') {
                return '支付宝';
            }
            if (_value === 'WeChat') {
                return '微信支付';
            }
            return _value;
        },
        
        _onSelectUsed: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.Used = _record.data.used;
            _view.reload();
        },
        _cleanUsed: function(_ctrl) {
            _ctrl.setValue('');
            let _controller = this;
            let _view = _controller.getView();
            if (_view.filter === null) _view.setFilter({});
            _view.filter.Used = null;
            _view.reload();
        },
        _onSelectInvoiced: function(_ctrl, _record) {
            let _controller = this;
            let _view = _controller.getView();
            _view.filter.Invoiced = _record.data.invoiced;
            _view.reload();
        },
        _cleanInvoiced: function(_ctrl) {
            _ctrl.setValue('');
            let _controller = this;
            let _view = _controller.getView();
            if (_view.filter === null) _view.setFilter({});
            _view.filter.Invoiced = null;
            _view.reload();
        },
        _onSelect: function(_ctrl, _record, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _ownerKvid=_record.data.Kvid;
        	delete _view.filter.OwnerKvid;
            _view.filter.OwnerKvid = _ownerKvid;
            _view.reload();
        },
        _cleanSelect: function(_ctrl) {
            _ctrl.setValue(null);
            let _me = this;
            let _view = _me.getView();
            if (_view.filter === null) _view.setFilter({});
            delete _view.filter.OwnerKvid;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            let _controller = this;
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _startTime = _refs.startTime.value;
            let _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.BeginTime;
            delete _view.filter.EndTime;
            _view.filter.BeginTime = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.EndTime = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        }
    }
});